import 'package:flutter/material.dart';
import 'package:adeptutils/adeptutils.dart';
import 'dart:async';

class DrawerWrapper {
  GlobalKey<ScaffoldState> scaffoldKey;
  State state;
  int selectedItemId;
  ListStack<int> visitedItemsId = ListStack();
}

class AdeptDrawer extends Drawer {
  final _wrapper = DrawerWrapper();
  set _scaffoldKey(GlobalKey<ScaffoldState> scaffoldKey) => _wrapper.scaffoldKey = scaffoldKey;
  GlobalKey<ScaffoldState> get _scaffoldKey => _wrapper.scaffoldKey;

  set _state(State state) => _wrapper.state = state;
  State get _state => _wrapper.state;

  set _selectedItemId(int selectedItemId) => _wrapper.selectedItemId = selectedItemId;
  int get selectedItemId => _wrapper.selectedItemId;

  set _visitedItemsId(ListStack<int> visitedItemsId) => _wrapper.visitedItemsId = visitedItemsId;
  ListStack<int> get visitedItemsId => _wrapper.visitedItemsId;

  AdeptDrawer({
    Key key,
    GlobalKey<ScaffoldState> scaffoldKey,
    State state,
    int defaultSelectedItemId,
    ListStack<int> defaultVisitedItemsId,
    double elevation = 16.0,
    Widget child,
    String semanticLabel,
  }) : super(
    key: key,
    elevation: elevation,
    child: child,
    semanticLabel: semanticLabel,
  ){
    _scaffoldKey = scaffoldKey;
    _state = state;
    if(defaultVisitedItemsId != null && defaultVisitedItemsId.isNotEmpty) {
      _visitedItemsId = defaultVisitedItemsId;
      if(visitedItemsId.isNotEmpty) _selectedItemId = visitedItemsId.peek();
    } else selectItem(defaultSelectedItemId);
  }

  bool get isDrawerOpen => _scaffoldKey != null ? _scaffoldKey.currentState.isDrawerOpen : false;
  bool get isEndDrawerOpen => _scaffoldKey != null ? _scaffoldKey.currentState.isEndDrawerOpen : false;
  void openDrawer() => _scaffoldKey != null ? _scaffoldKey.currentState.openDrawer() : null;
  void openEndDrawer() => _scaffoldKey != null ? _scaffoldKey.currentState.openEndDrawer() : null;
  void closeDrawer(BuildContext context) {
    if(context == null){
      if(_state != null) context = _state.context; else
      if(_scaffoldKey != null) context = _scaffoldKey.currentContext;
    }
    if(context != null && isDrawerOpen) Navigator.pop(context);
  }

  void selectItem(int selectedItemId, {bool repeated = false, bool update = true}){
    if(selectedItemId == null) return;

    if(!repeated && visitedItemsId.isNotEmpty) {
      int currentItemId = visitedItemsId.peek();
      if(currentItemId == selectedItemId) return;
    }
    visitedItemsId.push(_selectedItemId = selectedItemId);
    try{
      if(update && _state != null) _state.setState(() {});
    }catch(e){
      print(e);
    }

  }

  void _onBackVisit(){
    if(visitedItemsId.size > 1){
      visitedItemsId.pop();
      _selectedItemId = visitedItemsId.peek();
      try{
        if(_state != null) _state.setState(() {});
      }catch(e){
        print(e);
      }
    }
  }

  /// onBackPressed returns false to stop propagating the event,
  /// true in other cases
  Future<bool> onBackPressed({BuildContext context}) async{
    if(isDrawerOpen) {
      closeDrawer(context);
      return false;
    }
    _onBackVisit();
    return true;
  }

}